class InvalidConfigException(Exception):
    """Thrown when the configuration for a question is invalid"""


class EvaluationException(Exception):
    """Thrown when an excetption was thrown from either the evaluation of a code block or a user function call"""



import os

generated_quiz_path = os.getcwd() + "/generated_quizzes/"


# TODO: Evaluate generation of an xml title
class QuizBuilder(object):
    def __init__(self, xml_title):
        self.xml_title = xml_title.lower().replace(" ", "_")

    def setup(self):
        if not os.path.exists(generated_quiz_path):
            os.makedirs(generated_quiz_path)

        result = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
        result += "<quiz>\n" \
                  "<!-- question: 0  -->\n" \
                  "\t<question type=\"category\">\n" \
                  "\t\t<category>\n" \
                  "\t\t\t<text>Generated by QGen</text>\n" \
                  "\t\t</category>\n" \
                  "\t</question>\n"
        with open (generated_quiz_path + self.xml_title + ".xml", "w") as output_file:
            output_file.write(result)
        output_file.close()

    def build_question_for_xml(self, title=None, text=None, type=None):
        result = "\n\t<question type=\"" + type + "\">"
        result += "<name>\n" + "\t\t<text>" + title + "</text>\n" + "\t\t</name>"
        result += "<questiontext format=\"html\">\n" + "\t\t<text>" + text + "</text>\n" + "\t\t</questiontext>"
        result += "<generalfeedback format=\"html\">\n" \
                  "\t\t<text></text>\n" \
                  "\t\t</generalfeedback>\n" \
                  "\t\t<defaultgrade>1.0000000</defaultgrade>\n" \
                  "\t\t<penalty>0.1000000</penalty>\n" \
                  "\t\t<hidden>0</hidden>\n" \
                  "\t\t<single>true</single>\n" \
                  "\t\t<shuffleanswers>true</shuffleanswers>\n" \
                  "\t\t<answernumbering>abc</answernumbering>\n" \
                  "\t\t<correctfeedback format=\"html\">\n" \
                  "\t\t\t<text></text>\n" \
                  "\t\t</correctfeedback>\n" \
                  "\t\t<partiallycorrectfeedback format=\"html\">\n" \
                  "\t\t\t<text></text>\n" \
                  "\t\t</partiallycorrectfeedback>\n" \
                  "\t\t<incorrectfeedback format=\"html\">\n" \
                  "\t\t\t<text></text>\n" \
                  "\t\t</incorrectfeedback>\n"
        with open (generated_quiz_path + self.xml_title + ".xml", "a+") as output_file:
            output_file.write(result)
        output_file.close()

    def build_answer_for_xml(self, text, correct_feedback=None, fraction_amount="100"):
        if correct_feedback is None:
            correct_feedback = ''
        if fraction_amount == '':
            fraction_amount = "100"
        result = "\n\t\t<answer fraction=\"" + fraction_amount + "\" format=\"moodle_auto_format\">\n" + "\t\t\t<text>" + text + \
                 "</text>\n" \
                 "\t\t\t<feedback format=\"html\">\n" \
                 "\t\t\t\t<text>" + correct_feedback + "</text>\n" \
                                                       "\t\t\t</feedback>\n" \
                                                       "\t\t</answer>\n"
        with open(generated_quiz_path + self.xml_title + ".xml", "a+") as output_file:
            output_file.write(result)
        output_file.close()

    def build_distractor_for_xml(self, text, incorrect_feedback=None, fraction_amount="0"):
        if incorrect_feedback is None:
            incorrect_feedback = ''
        if fraction_amount == '':
            fraction_amount = "0"
        result = "\n\t\t<answer fraction=\"" + fraction_amount + "\" format=\"moodle_auto_format\">\n" + "\t\t\t<text>" + text + \
                 "</text>\n" \
                 "\t\t\t<feedback format=\"html\">\n" \
                 "\t\t\t\t<text>" + incorrect_feedback + "</text>\n" \
                                                         "\t\t\t</feedback>\n" \
                                                         "\t\t</answer>\n"
        with open(generated_quiz_path + self.xml_title + ".xml", "a+") as output_file:
            output_file.write(result)
        output_file.close()

    def build_question_end_tag(self):
        result = "\t</question>\n\n"
        with open(generated_quiz_path + self.xml_title + ".xml", "a+") as output_file:
            output_file.write(result)
        output_file.close()

    def build_quiz_end_tag(self):
        result = "</quiz>"
        with open(generated_quiz_path + self.xml_title + ".xml", "a+") as output_file:
            output_file.write(result)
        output_file.close()


import moodle_xml_builder as mxb
import markdown2
from random import choice
from qgen.qgen_exceptions import EvaluationException
from qgen.build_helpers import evaluate_functions, evaluate_blocks, validate_question

"""Functions to generate questions in different formats"""

container = "<![CDATA[%s]]>"


def gen_question(question):
    """Function to generate a question in plain text format"""

    params = {}
    for key, value in question.question_params.iteritems():
        try:

            params[key] = choice(value)
        except IndexError as e:
            raise EvaluationException("{0} - {1}".format(str(question.question_params), e.message))

    body_for_xml = question.body.format(**params)
    body_cache = body_for_xml
    try:
        body_for_xml = evaluate_functions(body_for_xml, params)
        body_for_xml = evaluate_blocks(body_for_xml, question.params_cache)
    except Exception as e:
        raise EvaluationException("{0} - {1}".format(body_cache, e.message))
    body_for_xml = container % markdown2.markdown(body_for_xml, extras=["fenced-code-blocks",
                                                                        "code-friendly"])  # body_for_xml will now be
    # of type unicode and not str

    body_for_xml = body_for_xml.replace("\n", "<br />")

    answers = []
    # Evaluate answers
    for answer in question.answers:
        answer = answer.format(**params)
        answer_cache = answer
        try:
            answer = evaluate_functions(answer, params)
            answer = evaluate_blocks(answer, question.params_cache)
        except Exception as e:
            raise EvaluationException("{0} - {1}".format(answer_cache, e.message))
        answer = container % markdown2.markdown(answer)
        answers.append(answer)

    distractors = []
    # Evaluate distractors
    for distractor in question.distractors:
        distractor = distractor.format(**params)
        distractor_cache = distractor
        try:
            distractor = evaluate_functions(distractor, params)
            distractor = evaluate_blocks(distractor, question.params_cache)
        except Exception as e:
            raise EvaluationException("{0} - {1}".format(distractor_cache, e.message))
        distractor = container % markdown2.markdown(distractor)
        distractors.append(distractor)

    return validate_question(body_for_xml, answers, distractors)


def gen_moodle_xml(question):
    """Function to generate the questions in Moodle XML format"""

    is_valid = gen_question(question)

    if is_valid:
        body_for_xml, answers, distractors = is_valid

        # Translate to Moodle XMl
        xml_builder = mxb.QuizBuilder(question.title)

        xml_builder.build_question_for_xml(question.title, body_for_xml, question.type)

        for answer in answers:
            xml_builder.build_answer_for_xml(answer, question.correct_feedback, question.correct_answer_weight)

        for distractor in distractors:
            xml_builder.build_distractor_for_xml(distractor, question.incorrect_feedback, question.incorrect_answer_weight)

        xml_builder.build_question_end_tag()
        return 1
    return 0



import getopt
import sys
import qgen
import os


def main(argv):
    input_file = ''
    output_format = "plain"
    count = 10
    try:
        opts, args = getopt.getopt(argv, "hi:f:c:", ["ifile=", "format=", "count="])
    except getopt.GetoptError:
        print 'qgen.py -i <input_file>'
        sys.exit(2)
    for opt, arg in opts:
        if opt == '-h':
            print 'Usage: qgen.py -i <input_file> -f [plain, moodle]'
            sys.exit()
        elif opt in ("-i", "--ifile"):
            input_file = arg
        elif opt in ("-f", "--format"):
            output_format = arg
        elif opt in ("-c", "--count"):
            count = arg
        else:
            print 'Usage: qgen.py -i <input_file> -f [plain, moodle] -c <count>'
            sys.exit()

    if input_file != '':
        if output_format == "moodle":
            file_path = "{0}/{1}".format(os.getcwd(),input_file)
            qgen.build_moodle_xml(file_path, int(count))
    else:
        print "Must specify input file"
        sys.exit()


if __name__ == "__main__":
    main(sys.argv[1:])


import yaml
from importlib import import_module
from built_in_functions import built_in_functions as functions
from qgen_exceptions import InvalidConfigException
import generators.moodle_xml_builder as mxb


class Question(object):
    COMPULSORY_CONFIGS = ['type', 'title', 'answer', 'body']  # Tags that need to be in the template

    """Class to model a generate questions"""

    def __init__(self, configuration, question_count=0):
        self.question_params = {}
        self.body, self.type, self.title, self.answers, self.distractors, self.correct_feedback, \
            self.incorrect_feedback, self.correct_answer_weight, \
            self.incorrect_answer_weight = self.add_config(configuration)
        self.question_count = question_count
        self.add_imports(configuration)
        if 'params' in configuration:
            self.build_question_params(configuration['params'])
        self.params_cache = self.question_params

    def add_config(self, config):
        body, q_type, title, answer = self.add_compulsory_config(config) if self.check_config(config) else None
        tags = ['correct_feedback', 'incorrect_feedback', 'correct_answer_weight', 'incorrect_answer_weight']
        q_distractor = config['distractor'] if 'distractor' in config else {}
        q_correct_feedback, q_incorrect_feedback, q_correct_answer_weight, q_incorrect_answer_weight = \
            map(lambda tag: config[tag] if tag in config else "", tags)
        return body, q_type, title, answer, q_distractor, q_correct_feedback, q_incorrect_feedback, \
            q_correct_answer_weight, q_incorrect_answer_weight

    @staticmethod
    def check_config(config):
        for name in Question.COMPULSORY_CONFIGS:
            if name not in config:
                raise InvalidConfigException(name + " is missing from the configuration.")
        return True

    @staticmethod
    def add_compulsory_config(config):
        return config['body'], config['type'], config['title'], config['answer']

    @staticmethod
    def add_imports(data):
        """Imports any external functions specified"""
        if 'imports' in data:
            imports = data['imports']
            for source in imports:
                try:
                    module = import_module(source)
                    for name, value in module.__dict__.iteritems():  # iterate through the module's attributes
                        if callable(value):  # check if callable for functions
                            functions[name] = value
                except AttributeError as e:
                    print e

    def build_question_params(self, params):
        """Binds the parameters to there actual values"""
        list_params = None
        for parameter_name, function_name in params.iteritems():
            for function_param, arguments in function_name.iteritems():
                function_arguments = {}
                if arguments is None:
                    function_arguments = {}
                elif type(arguments) != dict:
                    function_arguments['value'] = arguments
                else:
                    function_arguments = arguments
                function_arguments['count'] = self.question_count
                list_params = functions[function_param](function_arguments)
            self.question_params[parameter_name] = list_params


def build_moodle_xml(yml_file=None, number_of_questions=10):
    from generators.generate_moodle_xml import gen_moodle_xml
    with open(yml_file, 'r') as stream:
        try:
            threshold = 50
            dict_value = yaml.load(stream)
            title = dict_value.keys()[0]

            question = Question(dict_value[title], number_of_questions)
            print "Question {0}".format(title)
            xml_builder = mxb.QuizBuilder(title)
            xml_builder.setup()
            count = 0
            effort = 0
            while count < number_of_questions and effort < threshold:
                result = gen_moodle_xml(question)
                if result == 0:
                    effort += 1
                count += result
            xml_builder.build_quiz_end_tag()
            if effort == threshold:
                print "Could not generate {0} questions at best {1} questions were generated. " \
                      "Threshold is currently set to {2} increasing this value might increase the number of " \
                      "questions generated.".format(number_of_questions, count, threshold)
            print "-----------------------------"
        except yaml.YAMLError as exc:
            print(exc)



from qgen import functions
from random import choice
from qgen_exceptions import EvaluationException

"""Helper functions that are essential to the construction of questions"""


# to help decide if a question is valid
question_list = []


# TODO - check if any case where the parameters might be needed to evaluate the eval blocks
def evaluate_blocks(text, params):
    """Evaluates the code blocks delimited by a $ in a question's answer or distractor"""
    # TODO - pick a random value from a parameter
    def params_get(*args):
        return choice(params[choice(args)])

    # TODO - pick a random value from a random parameter except these specified
    def params_except(*args):
        keys = params.keys()
        for arg in args:
            keys.remove(arg)
        return choice(params[choice(keys)])
    text = text.replace("\$", "esCA")
    if text.count("$") % 2 != 0:
        raise EvaluationException("Incorrect number of $ found in a block")
    while "$" in text:
        start_index = text.index('$')
        end_index = text.index('$', start_index + 1) + 1

        # get code to be evaluated
        substr = text[start_index:end_index]

        # remove leading and trailing $
        eval_block = substr[1:-1]

        text = text.replace(substr, str(eval(eval_block)))
    text = text.replace("esCA", "$")
    return text


def evaluate_functions(text, params):
    """Evaluates the functions delimited by a @ in a question's answer or distractor"""
    text = text.replace("\@", "esAM")
    while "@" in text:
        start_index = text.index('@')
        end_index = text.index('@', start_index + 1) + 1
        substr = text[start_index:end_index]

        eval_block = substr[1:-1]
        # find function
        function_name = eval_block

        text = text.replace(substr, str(functions[function_name](params)))
    text = text.replace("esCA", "@")
    return text


def validate_question(body, answers, distractors):
    answers, distractors = validate_answer_distractor(answers, distractors)
    if answers:
        if not valid_question(body, answers, distractors):
            return None
        return body, answers, distractors
    else:
        return None


def valid_question(body, answers, distractors):
    description = (body, set(answers), set(distractors))

    if description in question_list:
        is_valid = False
    else:
        is_valid = True
        question_list.append(description)
    return is_valid


def validate_answer_distractor(answers, distractors):
    answer_list = []
    distractor_list = []
    for answer in answers:
        if answer not in answer_list:
            answer_list.append(answer)
    for distractor in distractors:
        if distractor not in distractor_list:
            distractor_list.append(distractor)
    return answer_list, distractor_list



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

setup(
    name='qgen',
    version='1.2',
    description='Question generating framework',
    long_description="Generate a random value of questions based on a template",
    author='Javon Davis, Howard Edwards, Alexander Nicholson',
    author_email='javonldavis14@gmail.com, howarde.jr@hotmail.com, alexj.nich@hotmail.com',
    url='https://github.com/JA-VON/QGenPY',  # TODO - add repository url
    license="MIT",
    packages=find_packages(exclude=('tests', 'docs' ))
)



import random

"""All built in functions source code should be written in this file"""


def randrange(values):
    return random.sample(range(values['start'], values['end']), values['end'] - values['start'])


def make_set(values):
    return values['value'].split(",")


def randint(values):
    return random.randint(values['start'], values['end'])


built_in_functions = {'randrange': randrange, 'set': make_set, 'randint': randint, }